from os import times
from pyapacheatlas.auth import ServicePrincipalAuthentication
from pyapacheatlas.core import AtlasEntity, AtlasProcess
import json
import argparse

from pyapacheatlas.core.client import PurviewClient
from pvclient.utils import excel
from pvclient.utils import account
from pvclient.client.purview_client import ExtendedPurviewClient

def get_configuration():
    with open('configs.json') as json_file:
        return json.load(json_file)

def build_service_principal():
    import os
    TENANT_ID = os.getenv('TENANT_ID')
    CLIENT_ID = os.getenv('CLIENT_ID')
    CLIENT_SECRET = os.getenv('CLIENT_SECRET')  
    return ServicePrincipalAuthentication(
            tenant_id = TENANT_ID,
            client_id = CLIENT_ID,
            client_secret = CLIENT_SECRET)

def upload_entities(client):
    entities = excel.parse_excel_file_to_entities()
    if not entities and len(entities) > 0:
        return client.upload_entities(entities)

def list_glossary_terms(client):
    glossary = client.get_glossary(name="Glossary", guid=None, detailed=True)
    try:
        return glossary["termInfo"]
    except KeyError:
        print("Your default glossary appears to be empty.")
        exit(3)

def build_all_terms_guid(client):
    termsInfo = list_glossary_terms(client)
    return [i for i in termsInfo]

def check_env_variables():
    import os
    if None in [os.getenv('TENANT_ID'), os.getenv('CLIENT_ID'), os.getenv('CLIENT_SECRET'), os.getenv('SUBSCRIPTION_ID')]:
        raise ValueError("Missing environment variables. Please refer to https://pypi.org/project/pvclient/")
        exit(1)
def main():
    parser = argparse.ArgumentParser(description='Interaction with Purview')
    parser.add_argument("--version", action='store_true', help="Version of purview command-line")
    parser.add_argument("--create-purview", action='store_true')
    parser.add_argument("--assign-roles", action='store_true')
    parser.add_argument("--delete-purview", action='store_true')
    parser.add_argument("--create-glossary", help="Create a glossary with name")
    parser.add_argument("--list-terms", action='store_true')
    parser.add_argument("--upload-entities", action='store_true')
    parser.add_argument("--import-terms")
    parser.add_argument("--delete-term", help="Delete a term from the default Glossary")
    parser.add_argument("--delete-all-terms", action="store_true", help="Delete all terms from the default Glossary")
    parser.add_argument("--list-term-templates", action="store_true", help="List all term templates from the default Glossary")
    parser.add_argument("--import-term-templates", help="Import term templates from a file to the default Glossary")
    parser.add_argument("--delete-term-templates", help="Delete all templates from a file from the default Glossary")
    args = parser.parse_args()

    if args.version:
        from pvclient import __version__
        print("Purview version {}".format(__version__))
        exit(0)
    check_env_variables()
    configs = get_configuration()
    # Create a client to connect to your service.
    client = ExtendedPurviewClient(
        account_name = configs["Purview-account-name"],
        authentication = build_service_principal()
    )

    if args.create_purview:
        account.create_purview()
        account.assign_roles()
    
    if args.assign_roles:
        account.assign_roles()

    if args.delete_purview:
        account.delete_purview()

    if args.upload_entities:
        upload_entities(client)
    
    if args.list_terms:
        termInfos = list_glossary_terms(client)
        print(json.dumps(termInfos, indent=2))
    
    if args.import_terms:
        results = client.import_terms(csv_path=args.import_terms, glossary_name="Glossary", glossary_guid=None)
        print(json.dumps(results, indent=2))

    if args.delete_term:
        res = client.delete_glossary_term(args.delete_term)
        print(json.dumps(res, indent=2))

    if args.delete_all_terms:
        term_guids = build_all_terms_guid(client)
        client.delete_all_terms(term_guids)
    
    if args.list_term_templates:
        res = client.get_all_term_templates()
        print(json.dumps(res, indent=2))
    
    if args.import_term_templates:
        with open(args.import_term_templates) as file:
            res = client.import_term_templates(json.load(file))
            print(json.dumps(res, indent=2))
    if args.delete_term_templates:
        with open(args.delete_term_templates) as file:
            res = client.delete_term_templates(json.load(file))
            print(json.dumps(res, indent=2))
    
    if args.create_glossary:
        res = client.create_glossary(args.create_glossary)
        print(json.dumps(res, indent=2))


if __name__ == "__main__":
    main()


from pyapacheatlas.readers import ExcelConfiguration,ExcelReader
import json
import os

def __get_configuration():
    with open('configs.json') as json_file:
        return json.load(json_file)

def make_excel_template(excel_file):
    if not os.path.exists(excel_file):
        ExcelReader.make_template(excel_file)

def parse_excel_file_to_entities():
    configs = __get_configuration()
    excel_file = configs["Purview-Excel-file"]
    
    # setup
    excel_config = ExcelConfiguration()
    excel_reader = ExcelReader(excel_config)

    # Create an empty excel template to be populated
    make_excel_template(excel_file)
    # This is just a helper to fill in some demo data
    # fill_in_workbook(file_path, excel_config)

    # Parses the excel file and creates a batch to upload
    entities = excel_reader.parse_bulk_entities(excel_file)

    # This is what is getting sent to your Atlas server
    # print(json.dumps(entities,indent=2))
    return entities
    

if __name__ == "__main__":
    pass


from azure.identity import ClientSecretCredential
from azure.mgmt import resource 
from azure.mgmt.resource import ResourceManagementClient
from azure.mgmt.purview import PurviewManagementClient
from azure.mgmt.purview.models import Identity, AccountSku, Account
from datetime import datetime, timedelta
import time
import json

from pyapacheatlas import auth

def __make_purview_client__():
    import os
    TENANT_ID = os.getenv('TENANT_ID')
    CLIENT_ID = os.getenv('CLIENT_ID')
    CLIENT_SECRET = os.getenv('CLIENT_SECRET')
    SUBSCRIPTION_ID = os.getenv('SUBSCRIPTION_ID')
    credential = ClientSecretCredential(
        client_id=CLIENT_ID,
        client_secret=CLIENT_SECRET, 
        tenant_id=TENANT_ID)
    return PurviewManagementClient(credential, subscription_id=SUBSCRIPTION_ID)
        

def assign_roles():
    import os
    SUBSCRIPTION_ID = os.getenv('SUBSCRIPTION_ID')
    INTERACTIVEGROUPS = os.getenv('INTERACTIVEGROUPS')
    cmd_parameters = ["az role assignment create --assignee '{group}'".format(group=auth['Yggdrasil-Data-Platform-Developers']),
        "--role '{role}'",
        "--subscription '{resource_group}'".format(resource_group=SUBSCRIPTION_ID)]
    cmd_template = " ".join(cmd_parameters)

    roles = ['Purview Data Curator', 'Purview Data Reader', 'Purview Data Source Administrator']
    
    for role in roles:
        print('Assigning {role} to {group}'.format(role=role, group=INTERACTIVEGROUPS))
        cmd = cmd_template.format(role=role)
        stream = os.popen(cmd)
        output = stream.readlines()
        print(output)
        time.sleep(5)
    


def create_purview():
    purview_client = __make_purview_client__()
    with open('configs.json') as json_file:
        configs = json.load(json_file)
        #Create a purview
        identity = Identity(type= "SystemAssigned")
        sku = AccountSku(name= 'Standard', capacity= 4)
        purview_resource = Account(identity=identity,sku=sku,location =configs['location'])
        rg_name = configs['Resource-group']
        purview_name = configs['Purview-account-name']

        try:
            pa = (purview_client.accounts.begin_create_or_update(rg_name, purview_name, purview_resource)).result()
            print("location:", pa.location, " Purview Account Name: ", purview_name, " Id: " , pa.id ," tags: " , pa.tags) 
        except:
            print("Error in submitting job to create account")
            print(pa)

        while (getattr(pa,'provisioning_state')) != "Succeeded" :
            pa = (purview_client.accounts.get(rg_name, purview_name))
            status = getattr(pa,'provisioning_state')
            print(status)
            if status == "Succeeded":
                assign_roles()
                break
            elif status == "Failed":
                break
            elif status == "Creating" or  status == "Updating":
                time.sleep(30)
            else:
                ValueError("Unhandled status")

def delete_purview():
    purview_client = __make_purview_client__()
    with open('configs.json') as json_file:
        configs = json.load(json_file)

    rg_name = configs['Resource-group']
    purview_name = configs['Purview-account-name']
    try:
        pa = purview_client.accounts.begin_delete(rg_name, purview_name).result()
        print(pa)
    except:
        print("Error in submitting job to create account")
        print(pa)


def main():
    create_purview()
    #delete_purview()



from pyapacheatlas.core import PurviewClient, AtlasException
import requests
import time
class ExtendedPurviewClient(PurviewClient):
    """
    Provides communication between your application and the Azure Purview
    service. Simplifies the requirements for knowing the endpoint url and
    requires only the Purview account name.

    :param str account_name:
        Your Purview account name.
    :param authentication:
        The method of authentication.
    :type authentication:
        :class:`~pyapacheatlas.auth.base.AtlasAuthBase`
    """

    def __init__(self, account_name, authentication=None):
        super().__init__(account_name, authentication)
    
    def delete_glossary_term(self, termguid):
        """
        Delete one or many termguid from your Apache Atlas server.

        :param termguid: The termguid you want to remove.
        :type guid: Union(str,list(str))
        :return:
            204 No Content OK. If glossary term delete was successful.
            404 Not Found
            If glossary term guid in invalid.
        :rtype: int
        """
        atlas_endpoint = self.endpoint_url + \
            "/glossary/term/{termguid}".format(termguid=termguid)
        delete_response = requests.delete(
            atlas_endpoint,
            headers=self.authentication.get_authentication_headers())

        try:
            delete_response.raise_for_status()
            return delete_response.status_code
        except requests.RequestException:
            if "errorCode" in delete_response:
                raise AtlasException(delete_response.text)
            else:
                raise requests.RequestException(delete_response.text)

    def delete_all_terms(self, termguids):
        """
        Delete one or many termguid from your Apache Atlas server.

        :param termguids: List of termguid you want to remove.
        :type termguids: Union(str,list(str))
        :return:
            204 No Content OK. If glossary term delete was successful.
            404 Not Found
            If glossary term guid in invalid.
        :rtype: int
        """
        for termguid in termguids:
            self.delete_glossary_term(termguid)
            time.sleep(3)

    def get_all_term_templates(self):
        atlas_endpoint = self.endpoint_url + "/types/typedefs"
        # TODO: Implement paging with offset and limit
        getResult = requests.get(
            atlas_endpoint,
            params={"type": "term_template"},
            headers=self.authentication.get_authentication_headers()
        )
        return self._handle_response(getResult)

    def import_term_templates(self, term_templates):
        atlas_endpoint = self.endpoint_url + "/types/typedefs"
        # TODO: Implement paging with offset and limit
        import json
        getResult = requests.post(
            atlas_endpoint,
            data=json.dumps(term_templates),
            headers=self.authentication.get_authentication_headers()
        )
        return self._handle_response(getResult)
    
    def delete_term_templates(self, term_templates):
        atlas_endpoint = self.endpoint_url + "/types/typedefs"
        # TODO: Implement paging with offset and limit
        import json
        delete_response = requests.delete(
            atlas_endpoint,
            data=json.dumps(term_templates),
            headers=self.authentication.get_authentication_headers()
        )
        try:
            delete_response.raise_for_status()
            return delete_response.status_code
        except requests.RequestException:
            if "errorCode" in delete_response:
                raise AtlasException(delete_response.text)
            else:
                raise requests.RequestException(delete_response.text)
    
    def create_glossary(self, name):
        atlas_endpoint = self.endpoint_url + "/glossary"
        import json
        res = requests.post(
            atlas_endpoint,
            data=json.dumps(
                {"qualifiedName": name, "name": name,
                "terms": []}),
            headers=self.authentication.get_authentication_headers()
        )
        return self._handle_response(res)